'''
AUTHORS:
- Alec Krueger, Texas A&M University, Geochemical and Environmental Research Group, alecmkrueger@tamu.edu.
- Xiao Ge, Texas A&M University, Geochemical and Environmental Research Group, gexiao@tamu.edu.
- Sakib Mahmud, Texas A&M University, Geochemical and Environmental Research Group, sakib@tamu.edu.


Module to ingest and process raw glider data into NetCDF files
'''
from .processor import Processor
from .variable import Variable
